#include <stdlib.h>
#include <sys/stat.h>

#include "http/res.h"

void http_res_set_body_buf(http_response *res, const char *body,
                           size_t body_len, bool owned) {
  res->body.type = http_body_buf;
  res->body.buf = (char *)body;
  res->body.expected_len = body_len;
  res->body.buf_owned = owned;
}

void http_res_set_body_file(http_response *res, const char *filename) {
  struct stat st;
  stat(filename, &st);

  // TODO error handling
  FILE *f = fopen(filename, "r");

  res->body.type = http_body_file;
  res->body.file = f;
  res->body.expected_len = st.st_size;
}

void http_res_add_header(http_response *res, http_header type,
                         const char *value, bool owned) {
  // Extend the header array
  if (res->headers.len == res->headers.cap) {
    http_response_header *new_arr = realloc(
        res->headers.arr, 2 * res->headers.cap * sizeof(http_response_header));

    if (new_arr == NULL) {
      // TODO error handling
      return;
    }

    res->headers.arr = new_arr;
    res->headers.cap *= 2;
  }

  res->headers.arr[res->headers.len].type = type;
  res->headers.arr[res->headers.len].value = value;
  res->headers.arr[res->headers.len].owned = owned;

  res->headers.len++;
}

void http_res_set_mime_type(http_response *res, http_mime_type mime_type) {
  http_res_add_header(res, http_header_content_type,
                      http_mime_type_names[mime_type][1], false);
}
