#include <stdio.h>

#include "http/types.h"
#include "http_loop.h"

http_loop_gctx *http_loop_gctx_init() {
  http_loop_gctx *gctx = calloc(sizeof(http_loop_gctx), 1);

  return gctx;
}

http_loop_ctx *http_loop_ctx_init(http_loop_gctx *g) {
  http_loop_ctx *ctx = calloc(sizeof(http_loop_ctx), 1);
  ctx->g = g;
  ctx->c = g->custom_ctx_init();

  // TODO error checking
  ctx->res.headers.arr = malloc(4 * sizeof(http_response_header));
  ctx->res.headers.cap = 4;

  return ctx;
}

void http_loop_ctx_free(http_loop_ctx *ctx) {
  http_loop_ctx_reset(ctx);
  ctx->g->custom_ctx_free(ctx->c);

  if (ctx->res.headers.cap > 0) {
    free(ctx->res.headers.arr);
  }

  free(ctx);
}

void http_loop_ctx_reset(http_loop_ctx *ctx) {
  ctx->route = NULL;
  ctx->current_step = 0;

  if (ctx->res.head != NULL) {
    free((void *)ctx->res.head);
    ctx->res.head = NULL;
  }

  http_body_reset(&ctx->req.body);
  http_body_reset(&ctx->res.body);

  for (size_t i = 0; i < ctx->res.headers.len; i++) {
    if (ctx->res.headers.arr[i].owned) {
      free((void *)ctx->res.headers.arr[i].value);
    }
  }

  // We don't set the cap as we can just keep the original array for the next
  // requests
  ctx->res.headers.len = 0;

  ctx->res.status = 0;
  ctx->res.head_len = 0;
  ctx->res.head_written = 0;

  ctx->g->custom_ctx_reset(ctx->c);
}
