#include <stdio.h>
#include <string.h>

#include "lnm/common.h"
#include "lsm/store.h"

#include "http/types.h"
#include "http_loop.h"
#include "lander.h"

const char lander_key_charset[] =
    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

http_route lander_routes[] = {
    {.type = http_route_literal,
     .method = http_get,
     .path = "/",
     .steps = {lander_get_index, NULL},
     .steps_res = {http_loop_step_write_header, http_loop_step_write_body,
                   NULL}},
    {
        .type = http_route_regex,
        .method = http_get,
        .path = "^/([^/]+)$",
        .steps = {lander_get_entry, NULL},
        .steps_res = {http_loop_step_write_header, lander_stream_body_to_client,
                      NULL},
    },
    {
        .type = http_route_regex,
        .method = http_delete,
        .path = "^/([^/]+)$",
        .steps = {http_loop_step_auth, lander_remove_entry, NULL},
        .steps_res = {http_loop_step_write_header, http_loop_step_write_body,
                      NULL},
    },
    {
        .type = http_route_regex,
        .method = http_post,
        .path = "^/s(l?)/([^/]*)$",
        .steps = {http_loop_step_auth, lander_post_redirect,
                  http_loop_step_body_to_buf, lander_post_redirect_body_to_attr,
                  NULL},
        .steps_res = {http_loop_step_write_header, http_loop_step_write_body,
                      NULL},
    },
    {.type = http_route_regex,
     .method = http_post,
     .path = "^/p(l?)/([^/]*)$",
     .steps = {http_loop_step_auth, http_loop_step_parse_content_length,
               lander_post_paste, lander_stream_body_to_entry, NULL},
     .steps_res = {http_loop_step_write_header, http_loop_step_write_body,
                   NULL}},
    {.type = http_route_regex,
     .method = http_post,
     .path = "^/f(l?)/([^/]*)$",
     .steps = {http_loop_step_auth, http_loop_step_parse_content_length,
               lander_post_file, lander_stream_body_to_entry, NULL},
     .steps_res = {http_loop_step_write_header, http_loop_step_write_body,
                   NULL}},
};

void *lander_gctx_init() { return calloc(1, sizeof(lander_gctx)); }

lnm_err lander_ctx_init(void **c_ctx, void *gctx) {
  lander_ctx *ctx = calloc(1, sizeof(lander_ctx));

  if (ctx == NULL) {
    return lnm_err_failed_alloc;
  }

  *c_ctx = ctx;

  return lnm_err_ok;
}

void lander_ctx_reset(lander_ctx *ctx) {
  if (ctx->entry != NULL) {
    lsm_entry_close(ctx->entry);

    ctx->entry = NULL;
  }
}

void lander_ctx_free(lander_ctx *ctx) { free(ctx); }

void lander_header_to_attr(http_loop_ctx *ctx, const char *header_name,
                           lander_attr_type attr_type) {
  lander_ctx *c_ctx = ctx->c;

  for (size_t i = 0; i < ctx->req.num_headers; i++) {
    const struct phr_header *header = &ctx->req.headers[i];

    if (strncmp(header->name, header_name, header->name_len) == 0) {
      if (header->value_len > 0) {
        lsm_str *value;
        lsm_str_init_copy_n(&value, (char *)header->value, header->value_len);

        lsm_entry_attr_insert(c_ctx->entry, attr_type, value);
      }

      return;
    }
  }
}

void lander_attr_to_header(lnm_http_loop_ctx *ctx, lander_attr_type attr_type,
                           lnm_http_header header_type) {
  lander_ctx *c_ctx = ctx->c;
  lsm_str *value;

  if (lsm_entry_attr_get(&value, c_ctx->entry, attr_type) == lsm_error_ok) {
    lnm_http_res_add_header_len(&ctx->res, header_type, (char *)lsm_str_ptr(value), lsm_str_len(value), false);
  }
}
